//================================================================
//=	  Tworzenie grafiki wektorowej i rastrowej
//=	  CPU = ATMega8515
//=   CLK = 8MHz
//================================================================

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "vector_disp.h"
#include "graphics.h"



//================================================================
//===== zmienne globalne =========================================
//================================================================
volatile unsigned char wsp[4];	 							// tablice przechowujaca wspolrzedne aktualnie rysowanej lini

const unsigned char ofset[4] = {155,136,108,89};			// przesuniecie kolejnych cyfr zegara cyfrowego (od prawej)

volatile unsigned char bufor_tekstu[64] = 					// tekst do wyswietlenia (koniecznie potega 2 !!!)
{
	"Osciloscope Tube Vector Display by Rusaa Design Center ver1.11 "
};		

volatile unsigned int linia_tekstu = 0;						// wskaznik przewijanego tekstu


extern volatile unsigned char czas[7]; 						// g,g,m,m,s,"godziny" i "minuty" (liczba stopni dla zegara analogowego)
//================================================================



//================================================================
//===== Inicjalizacja grafiki ====================================
//================================================================
void graph_init(void)										
{
	PORTC = 0;									// PC = port danych DACa = stan L
	DDRC = 0xFF;								// PC = wyjscie

	PORTE = (1 << 2);							// A0=0, A1=0, LDAC=1
	DDRE |= 0x07;								// PE0-PE2 = wyjscie

	PORTA |= (1 << 5);							// integratory zatrzymane, Z=OFF
	DDRA |= (1 << 5) | (1 << 6) | (1 << 7);		// PA5-PA7 = wyjscie
	
	TIMSK = (1 << 7);							// wlacz przerwanie od przepelnienia timera 1
	TCCR1A = 0;									// zwykly tryb timera 
	TCCR1B = 2;									// preskaler=8, wlacz timer 1
}
//================================================================



//================================================================
//===== Ladowanie przetwornika ===================================
//================================================================
void rysuj_linie(unsigned char ofset_x, unsigned char ofset_y)
{
	unsigned char i4;
	
	DAC_CONTROL = DAC_A;									// OutA = x
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[X] + ofset_x]);
	DAC_CONTROL = DAC_A_LATCH;								// zatrzasnij DACa
	
	DAC_CONTROL = DAC_C;									// OutC = y
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[Y] + ofset_y]);
	DAC_CONTROL = DAC_C_LATCH; 								// zatrzasnij DACa
	
	DAC_CONTROL = DAC_B;									// OutB = mod_x
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[MX]]);
	DAC_CONTROL = DAC_B_LATCH;								// zatrzasnij DACa

	DAC_CONTROL = DAC_D;									// OutD = mod_y
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[MY]]);
	DAC_CONTROL = DAC_D_LATCH;								// zatrzasnij DACa

	cli();													// wylacz przerwania
	STER = Z_ON;											// wlaczenie osi Z i integratorow
	for(i4=0;i4<20;i4++)
		delay2();											// czas na rysowanie (okolo 250 cykli)
	STER = BLANK;											// wygaszenie osi Z i reset integratorw
	sei();													// wlacz przerwania
}


void rysuj_punkt(void)
{
	unsigned char i4;
	
	DAC_CONTROL = DAC_A;									// OutA = x
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[X]]);
	DAC_CONTROL = DAC_A_LATCH;								// zatrzasnij DACa
	
	DAC_CONTROL = DAC_C;									// OutC = y
	DAC_DATA = pgm_read_byte_near(&wartosc[wsp[Y]]);
	DAC_CONTROL = DAC_C_LATCH; 								// zatrzasnij DACa

	cli();													// wylacz przerwania
	STER = Z_ON;											// wlaczenie osi Z i integratorow
	for(i4=0;i4<10;i4++)
		delay2();											// czas na rysowanie (okolo 130 cykli)
	STER = BLANK;											// wygaszenie osi Z i reset integratorw
	sei();													// wlacz przerwania
}

/*
void rysuj_linie_skalowana(unsigned char skala_x, unsigned char skala_y,unsigned char ofset_x, unsigned char ofset_y)
{
	unsigned char i4;
	//signed char ws=0;
	
	DAC_CONTROL = DAC_A;									// OutA = x
	DAC_DATA = pgm_read_byte_near(&wartosc[(wsp[X] / skala_x) + ofset_x]);
	DAC_CONTROL = DAC_A_LATCH;								// zatrzasnij DACa
	
	DAC_CONTROL = DAC_C;									// OutC = y
	DAC_DATA = pgm_read_byte_near(&wartosc[(wsp[Y] / skala_y) + ofset_y]);
	DAC_CONTROL = DAC_C_LATCH; 								// zatrzasnij DACa
	
	DAC_CONTROL = DAC_B;									// OutB = mod_x
	//ws = wsp[MX];
	DAC_DATA = pgm_read_byte_near(&wartosc[(128 + ((wsp[MX] - 128) / skala_x))]);
	DAC_CONTROL = DAC_B_LATCH;								// zatrzasnij DACa

	DAC_CONTROL = DAC_D;									// OutD = mod_y
	//ws = wsp[MY];
	DAC_DATA = pgm_read_byte_near(&wartosc[(128 + ((wsp[MY] - 128) / skala_x))]);
	DAC_CONTROL = DAC_D_LATCH;								// zatrzasnij DACa

	//STER = INT_ON;										// start integratorw
	STER = Z_ON;											// wlaczenie osi Z i integratorow
	for(i4=0;i4<20;i4++)
		delay2();											// czas na rysowanie (okolo 250 cykli)
	STER = BLANK;											// wygaszenie osi Z i reset integratorw
	//delay2();
}*/
//================================================================



//================================================================
//===== Wyswietlenie zegara ======================================
//================================================================
void zegar_analogowy(void)
{
	unsigned char i4;
	
	wsp[MX] = pgm_read_byte_near(&minutowa[0][czas[6]]);
	wsp[MY] = pgm_read_byte_near(&minutowa[1][czas[6]]);
	wsp[X] = wsp[Y] = 128;									// 128 = 2.5V => srodek tarczy
	
	rysuj_linie(0,0);										// narysuj wskazowke minutowa
	rysuj_linie(0,0);										// 2x dla wiekszej jasnosci
	
	wsp[MX] = pgm_read_byte_near(&godzinowa[0][czas[5]]);
	wsp[MY] = pgm_read_byte_near(&godzinowa[1][czas[5]]);
	wsp[X] = wsp[Y] = 128;									// 128 = 2.5V => srodek tarczy
	
	rysuj_linie(0,0);										// narysuj wskazowke godzinowa
	rysuj_linie(0,0);										// 2x dla wiekszej jasnosci
	
	for(i4=0;i4<21;i4++)
	{
		wsp[X] = pgm_read_byte_near(&tarcza[i4][X]);
		wsp[Y] = pgm_read_byte_near(&tarcza[i4][Y]);
		wsp[MX] = pgm_read_byte_near(&tarcza[i4][MX]);
		wsp[MY] = pgm_read_byte_near(&tarcza[i4][MY]);
		
		rysuj_linie(0,0);									// narysuj kolejny element tarczy
		rysuj_linie(0,0);									// 2x dla wiekszej jasnosci
	}	
	
	DAC_CONTROL = DAC_B;									// OutB = mod_x
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_B_LATCH;								// zatrzasnij DACa
	DAC_CONTROL = DAC_D;									// OutD = mod_y
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_D_LATCH;								// zatrzasnij DACa
	for(i4=0;i4<48;i4++)
	{
		wsp[X] = pgm_read_byte_near(&tarcza_kropki[i4][X]);
		wsp[Y] = pgm_read_byte_near(&tarcza_kropki[i4][Y]);
	
		rysuj_punkt();										// narysuj kolejna "kropke"
	}	
}


void zegar_cyfrowy(void) 									// h10 h1 m10 m1 
{
	signed char cyf,linia,ile_ln;
	
	for(cyf=3;cyf>(-1);cyf--)								// 4 kolejne cyfry od jednosci minut
	{
		ile_ln = pgm_read_byte_near(&cyfra[czas[cyf]][0][0]);
		for(linia=1;linia<ile_ln;linia++)					// kolejne linie skladowe cyfry
		{
			wsp[X] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][X]);
			wsp[Y] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][Y]);
			wsp[MX] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][MX]);
			wsp[MY] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][MY]);
		
			rysuj_linie(ofset[cyf],75);						// narysuj kolejna linie z cyfry
		}
	}	
	
	DAC_CONTROL = DAC_B;									// OutB = mod_x
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_B_LATCH;								// zatrzasnij DACa
	DAC_CONTROL = DAC_D;									// OutD = mod_y
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_D_LATCH;								// zatrzasnij DACa
	
	wsp[X] = pgm_read_byte_near(&dwukropek[0][X]);
	wsp[Y] = pgm_read_byte_near(&dwukropek[0][Y]);
	
	rysuj_punkt();											// narysuj gorna kropke
	
	wsp[X] = pgm_read_byte_near(&dwukropek[1][X]);
	wsp[Y] = pgm_read_byte_near(&dwukropek[1][Y]);
	
	rysuj_punkt();											// narysuj dolna kropke
}


/*void zegar_cyfrowy_skalowany(void) 						// h10 h1 m10 m1 
{
	signed char cyf,linia,ile_ln;
		
	for(cyf=3;cyf>(-1);cyf--)								// 4 kolejne cyfry od jednosci minut
	{
		ile_ln = pgm_read_byte_near(&cyfra[czas[cyf]][0][0]);
		for(linia=1;linia<ile_ln;linia++)					// kolejne linie skladowe cyfry
		{
			wsp[X] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][X]);
			wsp[Y] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][Y]);
			wsp[MX] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][MX]);
			wsp[MY] = pgm_read_byte_near(&cyfra[czas[cyf]][linia][MY]);
		
			rysuj_linie_skalowana(3,3,ofset[cyf],77);		// narysuj kolejna linie z cyfry
		}
	}	
	
	wsp[X] = pgm_read_byte_near(&dwukropek[0][X]);
	wsp[Y] = pgm_read_byte_near(&dwukropek[0][Y]);
	wsp[MX] = pgm_read_byte_near(&dwukropek[0][MX]);
	wsp[MY] = pgm_read_byte_near(&dwukropek[0][MY]);
	
	rysuj_linie_skalowana(2,2,0,77);						// narysuj gorna kropke
	
	wsp[X] = pgm_read_byte_near(&dwukropek[1][X]);
	wsp[Y] = pgm_read_byte_near(&dwukropek[1][Y]);
	wsp[MX] = pgm_read_byte_near(&dwukropek[1][MX]);
	wsp[MY] = pgm_read_byte_near(&dwukropek[1][MY]);
	
	rysuj_linie_skalowana(2,2,0,77);						// narysuj dolna kropke
}*/
//================================================================



//================================================================
//===== Grafika rastrowa =========================================
//================================================================
ISR(TIMER1_OVF_vect)										// przerwanie od timera 1
{
	linia_tekstu++;											// przesun tekst o 1 linie w lewo
	if(linia_tekstu > 383)									// przeszedl caly tekst
		linia_tekstu = 0;
}


void tekst_rastrowy(void)
{
	unsigned char i1,i2,i3,i5,linia,w_x,w_y,z1,z2,z3;
	
	DAC_CONTROL = DAC_B;									// OutB = mod_x
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_B_LATCH;								// zatrzasnij DACa
	DAC_CONTROL = DAC_D;									// OutD = mod_y
	DAC_DATA = 1;											// 128 -> bez przesuniecia
	DAC_CONTROL = DAC_D_LATCH;								// zatrzasnij DACa
	
	z1 = linia_tekstu / 6;
	z2 = linia_tekstu - (z1 * 6);
	z3 = 5 - z2;
	
	i5 = 0;
	
	for(i2=(z2+1);i2<5;i2++)								// 5 bajtow w znaku
	{
		linia = pgm_read_byte_near(&znak_raster[bufor_tekstu[(z1 & 63)]][i2]);
		
		w_x = 214 - (i5 * 3);								// ((5 lini + odstep = 6) * kolejny znak + linia w znaku) * co czwarty piksel
		
		for(i3=0;i3<7;i3++)									// 7 bitow w linii pionowej znaku
		{
			w_y = (i3 * 3) + 140;							// (linia pozioma znaku * co czwarty piksel) + polowa ekranu
			
			if((linia & 1) == 1)
			{
				wsp[X] = w_x;
				wsp[Y] = w_y;
	
				rysuj_punkt();								// narysuj kolejny piksel
			}
				
			linia >>= 1;									// przesun na kolejny piksel
		}
		i5++;
	}
	
	i5 = 0;
	
	for(i1=(z1 + 1);i1<(z1 + 9);i1++)						// wykonaj dla 10 znakow
	{
		for(i2=0;i2<5;i2++)									// 5 bajtow w znaku
		{
			linia = pgm_read_byte_near(&znak_raster[bufor_tekstu[(i1 & 63)]][i2]);
			
			w_x = 214 - (z3 * 3) - (((i5 * 6) + i2) * 3);	// ((5 lini + odstep = 6) * kolejny znak + linia w znaku) * co czwarty piksel
			
			for(i3=0;i3<7;i3++)								// 7 bitow w linii pionowej znaku
			{
				w_y = (i3 * 3) + 140;						// (linia pozioma znaku * co czwarty piksel) + polowa ekranu
				
				if((linia & 1) == 1)
				{
					wsp[X] = w_x;
					wsp[Y] = w_y;
	
					rysuj_punkt();							// narysuj kolejny piksel
				}
				
				linia >>= 1;								// przesun na kolejny piksel
			}
		}
		i5++;
	}
	
	i5 = 0;
	
	for(i2=0;i2<z2;i2++)									// 5 bajtow w znaku
	{
		linia = pgm_read_byte_near(&znak_raster[bufor_tekstu[((z1 + 9) & 63)]][i2]);
		
		w_x = 70 - (z3 * 3) - (i5 * 3);						// ((5 lini + odstep = 6) * kolejny znak + linia w znaku) * co czwarty piksel
		
		for(i3=0;i3<7;i3++)									// 7 bitow w linii pionowej znaku
		{
			w_y = (i3 * 3) + 140;							// (linia pozioma znaku * co czwarty piksel) + polowa ekranu
			
			if((linia & 1) == 1)
			{
				wsp[X] = w_x;
				wsp[Y] = w_y;
	
				rysuj_punkt();								// narysuj kolejny piksel
			}
				
			linia >>= 1;									// przesun na kolejny piksel
		}
		i5++;
	}
	
}
//================================================================

